@@ -46,7 +46,10 @@ class ENV_VARIABLES:
     FLINK_CONF_DIR = "FLINK_CONF_DIR"
     REQUESTS_VERIFY = "REQUESTS_VERIFY"
     REQUESTS_VERIFY_ENV_VAR = "REQUESTS_VERIFY"
+    DOMAIN_CA_TRUSTSTORE_ENV_VAR = "DOMAIN_CA_TRUSTSTORE"
     DOMAIN_CA_TRUSTSTORE_PEM_ENV_VAR = "DOMAIN_CA_TRUSTSTORE_PEM"
+    SECRETS_DIR_ENV_VAR = "SECRETS_DIR"
+
 
 class KAFKA_SSL_CONFIG:
     """
@@ -125,6 +128,7 @@ class SPARK_CONFIG:
     SPARK_OVERWRITE_MODE = "overwrite"
     SPARK_APPEND_MODE = "append"
     SPARK_WRITE_DELIMITER = "delimiter"
+    SPARK_INFER_SCHEMA = "inferSchema"
     SPARK_WRITE_HEADER = "header"
     SPARK_TF_CONNECTOR_RECORD_TYPE = "recordType"
     SPARK_TF_CONNECTOR_RECORD_TYPE_EXAMPLE = "Example"
@@ -189,6 +193,7 @@ class FEATURE_STORE:
     TRAINING_DATASET_TSV_FORMAT = "tsv"
     TRAINING_DATASET_PARQUET_FORMAT = "parquet"
     TRAINING_DATASET_TFRECORDS_FORMAT = "tfrecords"
+    TRAINING_DATASET_TFRECORD_FORMAT = "tfrecord"
     TRAINING_DATASET_AVRO_FORMAT = "avro"
     TRAINING_DATASET_ORC_FORMAT = "orc"
     TRAINING_DATASET_NPY_FORMAT = "npy"
@@ -210,6 +215,7 @@ class FEATURE_STORE:
         TRAINING_DATASET_CSV_FORMAT,
         TRAINING_DATASET_PARQUET_FORMAT,
         TRAINING_DATASET_TFRECORDS_FORMAT,
+        TRAINING_DATASET_TFRECORD_FORMAT,
         TRAINING_DATASET_NPY_FORMAT,
         TRAINING_DATASET_HDF5_FORMAT,
         TRAINING_DATASET_AVRO_FORMAT,
@@ -330,15 +336,11 @@ class REST_CONFIG:
     JSON_FEATURESTORE_SETTINGS_ENTITY_NAME_MAX_LENGTH = "featurestoreEntityNameMaxLength"
     JSON_FEATURESTORE_SETTINGS_ENTITY_DESCRIPTION_MAX_LENGTH = "featurestoreEntityDescriptionMaxLength"
     JSON_FEATURESTORE_SETTINGS_CACHED_FEATUREGROUP_DTO_TYPE = "cachedFeaturegroupDtoType"
-    JSON_FEATURESTORE_SETTINGS_CACHED_FEATUREGROUP_TYPE = "cachedFeaturegroupType"
-    JSON_FEATURESTORE_SETTINGS_EXTERNAL_TRAINING_DATASET_DTO_TYPE = "externalTrainingDatasetDtoType"
     JSON_FEATURESTORE_SETTINGS_EXTERNAL_TRAINING_DATASET_TYPE = "externalTrainingDatasetType"
-    JSON_FEATURESTORE_SETTINGS_FEATUREGROUP_TYPE = "featuregroupType"
     JSON_FEATURESTORE_SETTINGS_FEATURESTORE_REGEX = "featurestoreRegex"
     JSON_FEATURESTORE_SETTINGS_MAX_CORRELATIONS = "featurestoreStatisticsMaxCorrelations"
     JSON_FEATURESTORE_SETTINGS_HOPSFS_CONNECTOR_DTO_TYPE = "hopsfsConnectorDtoType"
     JSON_FEATURESTORE_SETTINGS_HOPSFS_CONNECTOR_TYPE = "hopsfsConnectorType"
-    JSON_FEATURESTORE_SETTINGS_HOPSFS_TRAINING_DATASET_DTO_TYPE = "hopsfsTrainingDatasetDtoType"
     JSON_FEATURESTORE_SETTINGS_HOPSFS_TRAINING_DATASET_TYPE = "hopsfsTrainingDatasetType"
     JSON_FEATURESTORE_SETTINGS_JDBC_CONNECTOR_DTO_TYPE = "jdbcConnectorDtoType"
     JSON_FEATURESTORE_SETTINGS_JDBC_CONNECTOR_TYPE = "jdbcConnectorType"
@@ -347,7 +349,6 @@ class REST_CONFIG:
         "jdbcStorageConnectorConnectionstringMaxLength"
     JSON_FEATURESTORE_SETTINGS_ON_DEMAND_FEATUREGROUP_DTO_TYPE = "onDemandFeaturegroupDtoType"
     JSON_FEATURESTORE_SETTINGS_ON_DEMAND_FEATUREGROUP_SQL_QUERY_MAX_LEN = "onDemandFeaturegroupSqlQueryMaxLength"
-    JSON_FEATURESTORE_SETTINGS_ON_DEMAND_FEATUREGROUP_TYPE = "onDemandFeaturegroupType"
     JSON_FEATURESTORE_SETTINGS_S3_CONNECTOR_DTO_TYPE = "s3ConnectorDtoType"
     JSON_FEATURESTORE_SETTINGS_S3_CONNECTOR_TYPE = "s3ConnectorType"
     JSON_FEATURESTORE_SETTINGS_S3_CONNECTOR_ACCESS_KEY_MAX_LEN = "s3StorageConnectorAccesskeyMaxLength"
@@ -370,17 +371,12 @@ class REST_CONFIG:
     JSON_FEATURESTORE_JOB_STATUS = "jobStatus"
     JSON_FEATURESTORE_JOB_NAME = "jobName"
     JSON_FEATURESTORE_JOB_ID = "jobId"
+    JSON_FEATURESTORE_LOCATION = "location"
 
-    JSON_FEATUREGROUP_ON_DEMAND_TYPE = "ON_DEMAND_FEATURE_GROUP"
-    JSON_FEATUREGROUP_CACHED_TYPE = "CACHED_FEATURE_GROUP"
-    JSON_FEATUREGROUP_HIVE_TABLE_TYPE = "hiveTableType"
-    JSON_FEATUREGROUP_INPUT_FORMAT = "inputFormat"
-    JSON_FEATUREGROUP_HDFS_STORE_PATHS = "hdfsStorePaths"
-    JSON_FEATUREGROUP_HIVE_TBL_ID = "hiveTableId"
     JSON_FEATUREGROUP_ON_DEMAND_QUERY = "query"
     JSON_FEATUREGROUP_JDBC_CONNECTOR_NAME = "jdbcConnectorName"
     JSON_FEATUREGROUP_JDBC_CONNECTOR_ID = "jdbcConnectorId"
-    JSON_FEATUREGROUP_TYPE = "featuregroupType"
+    JSON_FEATUREGROUP_TYPE = "type"
     JSON_FEATUREGROUP_NAME = "name"
     JSON_FEATUREGROUP_ID = "id"
     JSON_FEATUREGROUP_VERSION = "version"
@@ -392,13 +388,11 @@ class REST_CONFIG:
     JSON_FEATUREGROUP_FEATURE_CORRELATION = "featureCorrelationMatrix"
     JSON_FEATUREGROUP_DESC_STATS = "descriptiveStatistics"
     JSON_FEATUREGROUP_UPDATE_STATS = "updateStats"
-    JSON_FEATUREGROUP_HDFS_PATH = "hdfsStorePaths"
     JSON_FEATUREGROUP_FEATURES_HISTOGRAM = "featuresHistogram"
     JSON_FEATUREGROUP_FEATURES_CLUSTERS = "clusterAnalysis"
     JSON_FEATUREGROUPS = "featuregroups"
-    JSON_FEATUREGROUP_INODE_ID = "inodeId"
-    JSON_FEATUREGROUP_ONLINE = "onlineFeaturegroupEnabled"
-    JSON_FEATUREGROUP_ONLINE_DTO = "onlineFeaturegroupDTO"
+    JSON_FEATUREGROUP_ONLINE = "onlineEnabled"
+    JSON_FEATUREGROUP_HUDI = "hudiEnabled"
     JSON_FEATUREGROUP_FEATURE_HISTOGRAM_ENABLED = "featHistEnabled"
     JSON_FEATUREGROUP_FEATURE_CORRELATION_ENABLED = "featCorrEnabled"
     JSON_FEATUREGROUP_CLUSTER_ANALYSIS_ENABLED = "clusterAnalysisEnabled"
@@ -443,11 +437,9 @@ class REST_CONFIG:
     JSON_TRAINING_DATASET_EXTERNAL_TYPE = "EXTERNAL_TRAINING_DATASET"
     JSON_TRAINING_DATASET_HOPSFS_TYPE = "HOPSFS_TRAINING_DATASET"
     JSON_TRAINING_DATASET_TYPE = "trainingDatasetType"
-    JSON_TRAINING_DATASET_HOPSFS_CONNECTOR_NAME = "hopsfsConnectorName"
-    JSON_TRAINING_DATASET_HOPSFS_CONNECTOR_ID = "hopsfsConnectorId"
+    JSON_TRAINING_DATASET_CONNECTOR_NAME = "storageConnectorName"
+    JSON_TRAINING_DATASET_CONNECTOR_ID = "storageConnectorId"
     JSON_TRAINING_DATASET_SIZE = "size"
-    JSON_TRAINING_DATASET_S3_CONNECTOR_NAME = "s3ConnectorName"
-    JSON_TRAINING_DATASET_S3_CONNECTOR_ID = "s3ConnectorId"
     JSON_TRAINING_DATASET_ID = "id"
     JSON_TRAINING_DATASET_NAME = "name"
     JSON_TRAINING_DATASETS = "trainingDatasets"
@@ -498,13 +490,15 @@ class REST_CONFIG:
     HOPSWORKS_SERVING_RESOURCE = "serving"
     HOPSWORKS_INFERENCE_RESOURCE = "inference"
     HOPSWORKS_MODELS_RESOURCE = "models"
-    HOPSWORKS_USERS_RESOURCE = "serving"
+    HOPSWORKS_USERS_RESOURCE = "users"
+    HOPSWORKS_ADMIN_RESOURCE = "admin"
     HOPSWORKS_FEATURESTORES_STORAGE_CONNECTORS_RESOURCE = "storageconnectors"
     HOPSWORKS_ONLINE_FEATURESTORE_STORAGE_CONNECTOR_RESOURCE= "onlinefeaturestore"
-    HOPSWORKS_FEATUREGROUPS_XATTRS_RESOURCE = "xattrs"
+    HOPSWORKS_FEATURESTORE_TAGS_RESOURCE = "tags"
+    HOPSWORKS_VARIABLES_RESOURCE = "variables"
+    HOPSWORKS_ENDPOINT = "hopsworks_endpoint"
 
     HOPSWORKS_EXPERIMENTS_RESOURCE = "experiments"
-    HOPSWORKS_MODELS_RESOURCE = "models"
 
     HOPSWORKS_KAFKA_RESOURCE = "kafka"
     HOPSWORKS_TOPICS_RESOURCE = "topics"
@@ -514,8 +508,11 @@ class REST_CONFIG:
     HOPSWORKS_PROJECT_INFO_RESOURCE = "getProjectInfo"
     HOPSWORKS_JOBS_RESOURCE = "jobs"
     HOPSWORKS_EXECUTIONS_RESOURCE = "executions"
+    HOPSWORKS_DATASETS_RESOURCE = "dataset"
     HOPSWORKS_PROJECT_CREDENTIALS_RESOURCE = "credentials"
     HOPSWORKS_PROJECT_CLIENT = "client"
+    HOPSWORKS_AUTH_RESOURCE = "auth"
+    HOPSWORKS_AUTH_RESOURCE_REGISTER = "register"
 
     HOPSWORKS_XATTR_RESOURCE = "xattrs"
     HOPSWORKS_ELASTIC_RESOURCE = "elastic"
