@@ -2,11 +2,8 @@
 Utility functions for experiments
 """
 from __future__ import print_function
-try:
-    import __builtin__
-except ImportError:
-    # Python 3
-    import builtins as __builtin__
+
+import builtins as __builtin__
 
 import os
 import signal
@@ -26,10 +23,6 @@ from hops import constants
 from hops import devices
 from hops import util
 from hops import hdfs
-import urllib3
-urllib3.disable_warnings(urllib3.exceptions.SecurityWarning)
-
-import pydoop.hdfs
 
 logger_fd = None
 
@@ -39,7 +32,7 @@ def _init_logger(exec_logdir, role=None, index=None):
     """
 
     prefix = ''
-    if role != None and index != None:
+    if role is not None and index is not None:
         prefix = str(role) + '_' + str(index) + '_'
 
     logfile = exec_logdir + '/' + prefix + 'output.log'
@@ -154,7 +147,7 @@ def _upload_file_output(retval, hdfs_exec_logdir):
                 if os.path.exists(value): # absolute path
                     if hdfs.exists(hdfs_exec_logdir + '/' + value.split('/')[-1]):
                         hdfs.delete(hdfs_exec_logdir + '/' + value.split('/')[-1], recursive=False)
-                    pydoop.hdfs.put(value, hdfs_exec_logdir)
+                    hdfs.copy_to_hdfs(value, hdfs_exec_logdir)
                     os.remove(value)
                     hdfs_exec_logdir = hdfs.abs_path(hdfs_exec_logdir)
                     retval[metric_key] = hdfs_exec_logdir[len(hdfs.abs_path(hdfs.project_path())):] + '/' +  value.split('/')[-1]
@@ -162,7 +155,7 @@ def _upload_file_output(retval, hdfs_exec_logdir):
                     output_file = os.getcwd() + '/' + value
                     if hdfs.exists(hdfs_exec_logdir + '/' + value):
                         hdfs.delete(hdfs_exec_logdir + '/' + value, recursive=False)
-                    pydoop.hdfs.put(value, hdfs_exec_logdir)
+                    hdfs.copy_to_hdfs(value, hdfs_exec_logdir)
                     os.remove(output_file)
                     hdfs_exec_logdir = hdfs.abs_path(hdfs_exec_logdir)
                     retval[metric_key] = hdfs_exec_logdir[len(hdfs.abs_path(hdfs.project_path())):] + '/' +  output_file.split('/')[-1]
@@ -184,12 +177,6 @@ def _handle_return_simple(retval, hdfs_exec_logdir, logfile):
 
     """
     return_file = hdfs_exec_logdir + '/.outputs.json'
-    
-    if not retval:
-        if logfile is not None:
-            retval = {'log': logfile}
-            hdfs.dump(dumps(retval), return_file)
-        return
 
     _upload_file_output(retval, hdfs_exec_logdir)
 
@@ -200,7 +187,7 @@ def _handle_return_simple(retval, hdfs_exec_logdir, logfile):
         except:
             pass
 
-    retval['log'] = hdfs_exec_logdir.replace(hdfs.project_path(), '') + '/output.log'
+    retval['log'] = logfile
 
     hdfs.dump(dumps(retval), return_file)
 
@@ -242,6 +229,13 @@ def _cleanup(tensorboard, gpu_thread):
         print('Exception occurred while closing logger: {}'.format(err))
         pass
 
+    # Stop the gpu monitoring thread
+    try:
+        gpu_thread.do_run = False
+    except Exception as err:
+        print('Exception occurred while stopping GPU monitoring thread: {}'.format(err))
+        pass
+
 def _store_local_tensorboard(local_tb_path, hdfs_exec_logdir):
     """
 
@@ -255,7 +249,7 @@ def _store_local_tensorboard(local_tb_path, hdfs_exec_logdir):
     if os.path.exists(local_tb_path):
         tb_contents = os.listdir(local_tb_path)
         for entry in tb_contents:
-            pydoop.hdfs.put(local_tb_path + '/' + entry, hdfs_exec_logdir)
+            hdfs.copy_to_hdfs(local_tb_path + '/' + entry, hdfs_exec_logdir)
         try:
             shutil.rmtree(local_tb_path)
         except:
@@ -733,13 +727,14 @@ def _find_index(host_port, cluster_spec):
     return -1
 
 def _set_ml_id(app_id, run_id):
+    os.environ['HOME'] = os.getcwd()
     os.environ['ML_ID'] = str(app_id) + '_' + str(run_id)
 
 def _get_metric(return_dict, metric_key):
     if return_dict and metric_key:
         assert metric_key in return_dict.keys(), 'Supplied metric_key {} is not in returned dict {}'.format(metric_key, return_dict)
         return str(return_dict[metric_key])
-    elif return_dict and len(return_dict.keys()) == 1 and 'metric' in return_dict.keys():
+    elif return_dict is not None and len(return_dict.keys()) == 2 and 'metric' in return_dict.keys():
         return str(return_dict['metric'])
     else:
         return None
@@ -776,3 +771,11 @@ def build_parameters(map_fun, executor_num, args_dict):
         argIndex += 1
     param_string = param_string[:-1]
     return param_string, params, args
+
+def _create_experiment_dir(app_id, run_id):
+    experiment_path = _get_logdir(app_id, run_id)
+
+    if hdfs.exists(experiment_path):
+        hdfs.delete(experiment_path, recursive=True)
+
+    hdfs.mkdir(experiment_path)
\ No newline at end of file
