@@ -5,29 +5,6 @@ REST calls to Hopsworks Feature Store Service
 from hops import constants, util, hdfs
 from hops.exceptions import RestAPIError
 import json
-try:
-    from json.decoder import JSONDecodeError
-except ImportError:
-    JSONDecodeError = ValueError
-
-def _http(resource_url, headers=None, method=constants.HTTP_CONFIG.HTTP_GET, data=None):
-    response = util.send_request(method, resource_url, headers=headers, data=data)
-    try:
-        response_object = response.json()
-    except JSONDecodeError:
-        response_object = None
-
-    if (response.status_code // 100) != 2:
-        if response_object:
-            error_code, error_msg, user_msg = util._parse_rest_error(response_object)
-        else:
-            error_code, error_msg, user_msg = "", "", ""
-
-        raise RestAPIError("Could not execute HTTP request (url: {}), server response: \n "
-                           "HTTP code: {}, HTTP reason: {}, error code: {}, error msg: {}, user msg: {}".format(
-            resource_url, response.status_code, response.reason, error_code, error_msg, user_msg))
-
-    return response_object
 
 def _delete_table_contents(featuregroup_id, featurestore_id):
     """
@@ -55,16 +32,14 @@ def _delete_table_contents(featuregroup_id, featurestore_id):
                     str(featuregroup_id) + constants.DELIMITERS.SLASH_DELIMITER +
                     constants.REST_CONFIG.HOPSWORKS_FEATUREGROUP_CLEAR_RESOURCE)
     response = util.send_request(method, resource_url)
-    response_object = response.json()
 
     if response.status_code != 200:
+        response_object = response.json()
         error_code, error_msg, user_msg = util._parse_rest_error(response_object)
         raise RestAPIError("Could not clear featuregroup contents (url: {}), server response: \n "
                            "HTTP code: {}, HTTP reason: {}, error code: {}, error msg: {}, user msg: {}".format(
             resource_url, response.status_code, response.reason, error_code, error_msg, user_msg))
 
-    return response_object
-
 
 def _get_featurestores():
     """
@@ -129,24 +104,6 @@ def _get_featurestore_metadata(featurestore):
 
     return response_object
 
-def _get_project_info(project_name):
-    """
-    Makes a REST call to hopsworks to get all metadata of a project for the provided project.
-
-    Args:
-        :project_name: the name of the project
-
-    Returns:
-        JSON response
-
-    Raises:
-        :RestAPIError: if there was an error in the REST call to Hopsworks
-    """
-    return _http(constants.DELIMITERS.SLASH_DELIMITER +
-                 constants.REST_CONFIG.HOPSWORKS_REST_RESOURCE + constants.DELIMITERS.SLASH_DELIMITER +
-                 constants.REST_CONFIG.HOPSWORKS_PROJECT_RESOURCE + constants.DELIMITERS.SLASH_DELIMITER +
-                 constants.REST_CONFIG.HOPSWORKS_PROJECT_INFO_RESOURCE + constants.DELIMITERS.SLASH_DELIMITER +
-                 project_name)
 
 def _get_credentials(project_id):
     """
@@ -161,7 +118,7 @@ def _get_credentials(project_id):
     Raises:
         :RestAPIError: if there was an error in the REST call to Hopsworks
     """
-    return _http(constants.DELIMITERS.SLASH_DELIMITER +
+    return util.http(constants.DELIMITERS.SLASH_DELIMITER +
                  constants.REST_CONFIG.HOPSWORKS_REST_RESOURCE + constants.DELIMITERS.SLASH_DELIMITER +
                  constants.REST_CONFIG.HOPSWORKS_PROJECT_RESOURCE + constants.DELIMITERS.SLASH_DELIMITER +
                  project_id + constants.DELIMITERS.SLASH_DELIMITER +
@@ -215,7 +172,7 @@ def _create_featuregroup_rest(featuregroup, featurestore_id, description, featur
                               features_histogram_data, cluster_analysis_data, feature_corr_enabled,
                               featuregroup_desc_stats_enabled, features_histogram_enabled,
                               cluster_analysis_enabled, stat_columns, num_bins, num_clusters, corr_method,
-                              featuregroup_type, featuregroup_dto_type, sql_query, jdbc_connector_id, online_fg):
+                              featuregroup_type, sql_query, jdbc_connector_id, online_fg):
     """
     Sends a REST call to hopsworks to create a new featuregroup with specified metadata
 
@@ -239,7 +196,6 @@ def _create_featuregroup_rest(featuregroup, featurestore_id, description, featur
         :num_clusters: the number of clusters to use for cluster analysis
         :corr_method: the method to compute feature correlation with (pearson or spearman)
         :featuregroup_type: type of the featuregroup (on-demand or cached)
-        :featuregroup_dto_type: type of the JSON DTO for the backend
         :sql_query: SQL Query for On-demand feature groups
         :jdbc_connector_id: id of the jdbc_connector for on-demand feature groups
         :online_fg: whether online feature serving should be enabled for the feature group
@@ -259,8 +215,7 @@ def _create_featuregroup_rest(featuregroup, featurestore_id, description, featur
                      constants.REST_CONFIG.JSON_FEATUREGROUP_DESC_STATS: featuregroup_desc_stats_data,
                      constants.REST_CONFIG.JSON_FEATUREGROUP_FEATURES_HISTOGRAM: features_histogram_data,
                      constants.REST_CONFIG.JSON_FEATUREGROUP_FEATURES_CLUSTERS: cluster_analysis_data,
-                     constants.REST_CONFIG.JSON_TYPE: featuregroup_dto_type,
-                     constants.REST_CONFIG.JSON_FEATURESTORE_SETTINGS_FEATUREGROUP_TYPE: featuregroup_type,
+                     constants.REST_CONFIG.JSON_FEATUREGROUP_TYPE: featuregroup_type,
                      constants.REST_CONFIG.JSON_FEATUREGROUP_ONLINE: online_fg,
                      constants.REST_CONFIG.JSON_FEATUREGROUP_CLUSTER_ANALYSIS_ENABLED: cluster_analysis_enabled,
                      constants.REST_CONFIG.JSON_FEATUREGROUP_DESCRIPTIVE_STATISTICS_ENABLED: featuregroup_desc_stats_enabled,
@@ -271,7 +226,7 @@ def _create_featuregroup_rest(featuregroup, featurestore_id, description, featur
                      constants.REST_CONFIG.JSON_FEATUREGROUP_NUM_CLUSTERS: num_clusters,
                      constants.REST_CONFIG.JSON_FEATUREGROUP_CORR_METHOD: corr_method
                      }
-    if featuregroup_type == constants.REST_CONFIG.JSON_FEATUREGROUP_ON_DEMAND_TYPE:
+    if featuregroup_type == "onDemandFeaturegroupDTO":
         json_contents[constants.REST_CONFIG.JSON_FEATUREGROUP_ON_DEMAND_QUERY] = sql_query
         json_contents[constants.REST_CONFIG.JSON_FEATUREGROUP_JDBC_CONNECTOR_ID] = jdbc_connector_id
     json_embeddable = json.dumps(json_contents, allow_nan=False)
@@ -299,8 +254,7 @@ def _create_featuregroup_rest(featuregroup, featurestore_id, description, featur
 def _update_featuregroup_stats_rest(featuregroup_id, featurestore_id, feature_corr,
                                     featuregroup_desc_stats_data, features_histogram_data, cluster_analysis_data,
                                     descriptive_statistics, feature_correlation, feature_histograms, cluster_analysis,
-                                    stat_columns, num_bins, num_clusters, corr_method, featuregroup_type,
-                                    featuregroup_dto_type, jobs):
+                                    stat_columns, num_bins, num_clusters, corr_method, featuregroup_type, jobs):
     """
     Makes a REST call to hopsworks appservice for updating the statistics of a particular featuregroup
 
@@ -320,7 +274,6 @@ def _update_featuregroup_stats_rest(featuregroup_id, featurestore_id, feature_co
         :num_clusters: the number of clusters to use for the cluster analysis of the featuregroup
         :corr_method: the correlation method to use for the correlation analysis of the featuregroup
         :featuregroup_type: type of the featuregroup (on-demand or cached)
-        :featuregroup_dto_type: type of the JSON DTO for the backend
         :jobs: a list of jobs for updating the feature group stats
 
     Returns:
@@ -342,8 +295,7 @@ def _update_featuregroup_stats_rest(featuregroup_id, featurestore_id, feature_co
                      constants.REST_CONFIG.JSON_FEATUREGROUP_NUM_BINS: num_bins,
                      constants.REST_CONFIG.JSON_FEATUREGROUP_NUM_CLUSTERS: num_clusters,
                      constants.REST_CONFIG.JSON_FEATUREGROUP_CORR_METHOD: corr_method,
-                     constants.REST_CONFIG.JSON_TYPE: featuregroup_dto_type,
-                     constants.REST_CONFIG.JSON_FEATURESTORE_SETTINGS_FEATUREGROUP_TYPE: featuregroup_type,
+                     constants.REST_CONFIG.JSON_TYPE: featuregroup_type,
                      }
     json_embeddable = json.dumps(json_contents, allow_nan=False)
     headers = {constants.HTTP_CONFIG.HTTP_CONTENT_TYPE: constants.HTTP_CONFIG.HTTP_APPLICATION_JSON}
@@ -380,7 +332,7 @@ def _update_featuregroup_stats_rest(featuregroup_id, featurestore_id, feature_co
 
 
 def _enable_featuregroup_online_rest(featuregroup, featuregroup_version, featuregroup_id, featurestore_id,
-                                     featuregroup_dto_type, featuregroup_type, features_schema):
+                                     featuregroup_type, features_schema):
     """
     Makes a REST call to hopsworks appservice for enabling online serving of a feature group (Create MySQL Table)
 
@@ -388,7 +340,6 @@ def _enable_featuregroup_online_rest(featuregroup, featuregroup_version, feature
         :featuregroup_id: id of the featuregroup
         :featurestore_id: id of the featurestore where the featuregroup resides
         :featuregroup_type: type of the featuregroup (on-demand or cached)
-        :featuregroup_dto_type: type of the JSON DTO for the backend
         :features_schema: the schema of the featuregroup (to create the MySQL table)
         :featuregroup: name of the featuregroup
         :featuregroup_version: version of the featuregroup
@@ -399,8 +350,7 @@ def _enable_featuregroup_online_rest(featuregroup, featuregroup_version, feature
     Raises:
         :RestAPIError: if there was an error in the REST call to Hopsworks
     """
-    json_contents = {constants.REST_CONFIG.JSON_TYPE: featuregroup_dto_type,
-                     constants.REST_CONFIG.JSON_FEATURESTORE_SETTINGS_FEATUREGROUP_TYPE: featuregroup_type,
+    json_contents = {constants.REST_CONFIG.JSON_TYPE: featuregroup_type,
                      constants.REST_CONFIG.JSON_FEATUREGROUP_FEATURES: features_schema,
                      constants.REST_CONFIG.JSON_FEATUREGROUP_NAME: featuregroup,
                      constants.REST_CONFIG.JSON_FEATUREGROUP_VERSION: featuregroup_version
@@ -436,7 +386,7 @@ def _enable_featuregroup_online_rest(featuregroup, featuregroup_version, feature
 
 
 def _disable_featuregroup_online_rest(featuregroup_name, featuregroup_version, featuregroup_id, featurestore_id,
-                                      featuregroup_dto_type, featuregroup_type):
+                                      featuregroup_type):
     """
     Makes a REST call to hopsworks appservice for disable online serving of a feature group (Drop MySQL table)
 
@@ -446,7 +396,6 @@ def _disable_featuregroup_online_rest(featuregroup_name, featuregroup_version, f
         :featuregroup_id: id of the featuregroup
         :featurestore_id: id of the featurestore where the featuregroup resides
         :featuregroup_type: type of the featuregroup (on-demand or cached)
-        :featuregroup_dto_type: type of the JSON DTO for the backend
 
     Returns:
         The REST response
@@ -454,8 +403,7 @@ def _disable_featuregroup_online_rest(featuregroup_name, featuregroup_version, f
     Raises:
         :RestAPIError: if there was an error in the REST call to Hopsworks
     """
-    json_contents = {constants.REST_CONFIG.JSON_TYPE: featuregroup_dto_type,
-                     constants.REST_CONFIG.JSON_FEATURESTORE_SETTINGS_FEATUREGROUP_TYPE: featuregroup_type,
+    json_contents = {constants.REST_CONFIG.JSON_TYPE: featuregroup_type,
                      constants.REST_CONFIG.JSON_FEATUREGROUP_NAME: featuregroup_name,
                      constants.REST_CONFIG.JSON_FEATUREGROUP_VERSION: featuregroup_version
                      }
@@ -491,8 +439,8 @@ def _disable_featuregroup_online_rest(featuregroup_name, featuregroup_version, f
 def _create_training_dataset_rest(training_dataset, featurestore_id, description, training_dataset_version,
                                   data_format, jobs, features_schema_data,
                                   feature_corr_data, training_dataset_desc_stats_data, features_histogram_data,
-                                  cluster_analysis_data, training_dataset_type, training_dataset_dto_type,
-                                  settings, hopsfs_connector_id = None, s3_connector_id = None):
+                                  cluster_analysis_data, training_dataset_type, settings,
+                                  connector_id = None, path = None):
     """
     Makes a REST request to hopsworks for creating a new training dataset
 
@@ -509,10 +457,10 @@ def _create_training_dataset_rest(training_dataset, featurestore_id, description
         :training_dataset_desc_stats_data: json-string with the descriptive statistics of the training dataset
         :features_histogram_data: list of json-strings with histogram data for the features in the training dataset
         :training_dataset_type: type of the training dataset (external or hopsfs)
-        :training_dataset_dto_type: type of the JSON DTO for the backend
         :hopsfs_connector_id: id of the connector for a hopsfs training dataset
         :s3_connector_id: id of the connector for a s3 training dataset
         :settings: featurestore settings
+        :path: the path within the storage connector where to save the training dataset
 
     Returns:
         the HTTP response
@@ -530,13 +478,11 @@ def _create_training_dataset_rest(training_dataset, featurestore_id, description
                      constants.REST_CONFIG.JSON_TRAINING_DATASET_FEATURES_HISTOGRAM: features_histogram_data,
                      constants.REST_CONFIG.JSON_TRAINING_DATASET_CLUSTERS: cluster_analysis_data,
                      constants.REST_CONFIG.JSON_TRAINING_DATASET_FORMAT: data_format,
-                     constants.REST_CONFIG.JSON_TYPE: training_dataset_dto_type,
-                     constants.REST_CONFIG.JSON_FEATURESTORE_SETTINGS_TRAINING_DATASET_TYPE: training_dataset_type
+                     constants.REST_CONFIG.JSON_FEATURESTORE_SETTINGS_TRAINING_DATASET_TYPE: training_dataset_type,
+                     constants.REST_CONFIG.JSON_FEATURESTORE_LOCATION: path,
+                     constants.REST_CONFIG.JSON_TRAINING_DATASET_CONNECTOR_ID: connector_id 
                      }
-    if training_dataset_type == settings.external_training_dataset_type:
-        json_contents[constants.REST_CONFIG.JSON_TRAINING_DATASET_S3_CONNECTOR_ID] = s3_connector_id
-    if training_dataset_type == settings.hopsfs_training_dataset_type:
-        json_contents[constants.REST_CONFIG.JSON_TRAINING_DATASET_HOPSFS_CONNECTOR_ID] = hopsfs_connector_id
+
     json_embeddable = json.dumps(json_contents, allow_nan=False)
     headers = {constants.HTTP_CONFIG.HTTP_CONTENT_TYPE: constants.HTTP_CONFIG.HTTP_APPLICATION_JSON}
     method = constants.HTTP_CONFIG.HTTP_POST
@@ -561,7 +507,7 @@ def _create_training_dataset_rest(training_dataset, featurestore_id, description
 
 def _update_training_dataset_stats_rest(
         training_dataset_id, featurestore_id, feature_corr_data, featuregroup_desc_stats_data,
-        features_histogram_data, cluster_analysis_data, training_dataset_type, training_dataset_dto_type, jobs):
+        features_histogram_data, cluster_analysis_data, training_dataset_type, jobs):
     """
     A helper function that makes a REST call to hopsworks for updating the stats and schema metadata about a
     training dataset
@@ -574,7 +520,6 @@ def _update_training_dataset_stats_rest(
         :features_histogram_data: list of json-strings with histogram data for the features in the training dataset
         :cluster_analysis_data: the clusters from cluster analysis on the dataset
         :training_dataset_type: type of the training dataset (external or hopsfs)
-        :training_dataset_dto_type: type of the JSON DTO for the backend
         :jobs: list of jobs for updating the training dataset stats
 
     Returns:
@@ -587,7 +532,6 @@ def _update_training_dataset_stats_rest(
                      constants.REST_CONFIG.JSON_FEATUREGROUP_DESC_STATS: featuregroup_desc_stats_data,
                      constants.REST_CONFIG.JSON_FEATUREGROUP_FEATURES_HISTOGRAM: features_histogram_data,
                      constants.REST_CONFIG.JSON_TRAINING_DATASET_CLUSTERS: cluster_analysis_data,
-                     constants.REST_CONFIG.JSON_TYPE: training_dataset_dto_type,
                      constants.REST_CONFIG.JSON_FEATURESTORE_SETTINGS_TRAINING_DATASET_TYPE: training_dataset_type,
                      constants.REST_CONFIG.JSON_TRAINING_DATASET_JOBS: _pre_process_jobs_list(jobs)}
     json_embeddable = json.dumps(json_contents, allow_nan=False)
@@ -690,8 +634,7 @@ def _get_training_dataset_rest(training_dataset_id, featurestore_id):
 
 def _sync_hive_table_with_featurestore_rest(featuregroup, featurestore_id, description, featuregroup_version, jobs,
                                             feature_corr_data, featuregroup_desc_stats_data,
-                                            features_histogram_data, cluster_analysis_data, featuregroup_type,
-                                            featuregroup_dto_type):
+                                            features_histogram_data, cluster_analysis_data, featuregroup_type):
     """
     Sends a REST call to hopsworks to synchronize a Hive table with the feature store
 
@@ -706,7 +649,6 @@ def _sync_hive_table_with_featurestore_rest(featuregroup, featurestore_id, descr
         :features_histogram_data: list of json-strings with histogram data for the features in the featuregroup
         :cluster_analysis_data: cluster analysis for the featuregroup
         :featuregroup_type: type of the featuregroup (on-demand or cached)
-        :featuregroup_dto_type: type of the JSON DTO for the backend
 
     Returns:
         The HTTP response
@@ -722,8 +664,7 @@ def _sync_hive_table_with_featurestore_rest(featuregroup, featurestore_id, descr
                      constants.REST_CONFIG.JSON_FEATUREGROUP_DESC_STATS: featuregroup_desc_stats_data,
                      constants.REST_CONFIG.JSON_FEATUREGROUP_FEATURES_HISTOGRAM: features_histogram_data,
                      constants.REST_CONFIG.JSON_FEATUREGROUP_FEATURES_CLUSTERS: cluster_analysis_data,
-                     constants.REST_CONFIG.JSON_TYPE: featuregroup_dto_type,
-                     constants.REST_CONFIG.JSON_FEATURESTORE_SETTINGS_FEATUREGROUP_TYPE: featuregroup_type
+                     constants.REST_CONFIG.JSON_TYPE: featuregroup_type,
                      }
 
     json_embeddable = json.dumps(json_contents)
@@ -779,53 +720,55 @@ def _get_online_featurestore_jdbc_connector_rest(featurestore_id):
     return response_object
 
 
-def _add_metadata(featurestore_id, featuregroup_id, name, value):
+def _add_tag(featurestore_id, id, tag, value, resource):
     """
-    Makes a REST call to Hopsworks to attach extended metadata to a featuregroup
+    Makes a REST call to Hopsworks to attach tags to a featuregroup or training dataset
 
     Args:
         :featurestore_id: the id of the featurestore
-        :featuregroup_id: the id of the featuregroup
-        :name: the name of the extended attribute
-        :value: the value of the extended attribute
+        :id: the id of the featuregroup or training dataset
+        :tag: name of the tag to attach
+        :value: value of the tag
+        :resource: featuregroup or training dataset resource
 
     Returns:
         None
 
     """
     method = constants.HTTP_CONFIG.HTTP_PUT
-    headers = {constants.HTTP_CONFIG.HTTP_CONTENT_TYPE: constants.HTTP_CONFIG.HTTP_APPLICATION_JSON}
-    data = json.dumps({name: str(value)})
     resource_url = (constants.DELIMITERS.SLASH_DELIMITER +
                     constants.REST_CONFIG.HOPSWORKS_REST_RESOURCE + constants.DELIMITERS.SLASH_DELIMITER +
                     constants.REST_CONFIG.HOPSWORKS_PROJECT_RESOURCE + constants.DELIMITERS.SLASH_DELIMITER +
                     hdfs.project_id() + constants.DELIMITERS.SLASH_DELIMITER +
                     constants.REST_CONFIG.HOPSWORKS_FEATURESTORES_RESOURCE + constants.DELIMITERS.SLASH_DELIMITER +
                     str(featurestore_id) + constants.DELIMITERS.SLASH_DELIMITER +
-                    constants.REST_CONFIG.HOPSWORKS_FEATUREGROUPS_RESOURCE +
-                    constants.DELIMITERS.SLASH_DELIMITER + str(featuregroup_id) + constants.DELIMITERS.SLASH_DELIMITER +
-                    constants.REST_CONFIG.HOPSWORKS_FEATUREGROUPS_XATTRS_RESOURCE + constants.DELIMITERS.SLASH_DELIMITER +
-                    name)
-    response = util.send_request(method, resource_url, data=data, headers=headers)
+                    resource +
+                    constants.DELIMITERS.SLASH_DELIMITER + str(id) + constants.DELIMITERS.SLASH_DELIMITER +
+                    constants.REST_CONFIG.HOPSWORKS_FEATURESTORE_TAGS_RESOURCE + constants.DELIMITERS.SLASH_DELIMITER +
+                    tag)
+    if value is not None:
+        resource_url = resource_url + "?value=" + value
+
+    response = util.send_request(method, resource_url)
     response_object = response.json()
     if response.status_code >= 400:
         error_code, error_msg, user_msg = util._parse_rest_error(response_object)
-        raise RestAPIError("Could not attach extened attributes to a featuregroup (url: {}), server response: \n " \
+        raise RestAPIError("Could not attach tags (url: {}), server response: \n " \
                            "HTTP code: {}, HTTP reason: {}, error code: {}, error msg: {}, user msg: {}".format(
             resource_url, response.status_code, response.reason, error_code, error_msg, user_msg))
 
 
-def _get_metadata(featurestore_id, featuregroup_id, name=None):
+def _get_tags(featurestore_id, id, resource):
     """
-    Makes a REST call to Hopsworks to get extended metadata attached to a featuregroup
+    Makes a REST call to Hopsworks to get tags attached to a featuregroup or training dataset
 
     Args:
         :featurestore_id: the id of the featurestore
-        :featuregroup_id: the id of the featuregroup
-        :name: the name of the extended attribute
+        :id: the id of the featuregroup or training dataset
+        :resource: featuregroup or training dataset resource
 
     Returns:
-        A dictionary containing the extended metadata
+        A dictionary containing the tags
 
     """
     method = constants.HTTP_CONFIG.HTTP_GET
@@ -836,57 +779,105 @@ def _get_metadata(featurestore_id, featuregroup_id, name=None):
                     hdfs.project_id() + constants.DELIMITERS.SLASH_DELIMITER +
                     constants.REST_CONFIG.HOPSWORKS_FEATURESTORES_RESOURCE + constants.DELIMITERS.SLASH_DELIMITER +
                     str(featurestore_id) + constants.DELIMITERS.SLASH_DELIMITER +
-                    constants.REST_CONFIG.HOPSWORKS_FEATUREGROUPS_RESOURCE +
-                    constants.DELIMITERS.SLASH_DELIMITER + str(featuregroup_id) + constants.DELIMITERS.SLASH_DELIMITER +
-                    constants.REST_CONFIG.HOPSWORKS_FEATUREGROUPS_XATTRS_RESOURCE)
-    if name is not None:
-        resource_url += constants.DELIMITERS.SLASH_DELIMITER + name
+                    resource +
+                    constants.DELIMITERS.SLASH_DELIMITER + str(id) + constants.DELIMITERS.SLASH_DELIMITER +
+                    constants.REST_CONFIG.HOPSWORKS_FEATURESTORE_TAGS_RESOURCE)
     response = util.send_request(method, resource_url, headers=headers)
     response_object = response.json()
     if response.status_code >= 400:
         error_code, error_msg, user_msg = util._parse_rest_error(response_object)
-        raise RestAPIError("Could not get extened attributes for a featuregroup (url: {}), server response: \n " \
+        raise RestAPIError("Could not get tags (url: {}), server response: \n " \
                            "HTTP code: {}, HTTP reason: {}, error code: {}, error msg: {}, user msg: {}".format(
             resource_url, response.status_code, response.reason, error_code, error_msg, user_msg))
 
-    results = {}
-    for item in response_object["items"]:
-        results[item["name"]] = item["value"]
-    return results
+    tags = {}
+    if "items" in response_object:
+        for tag in response_object["items"]:
+            tags[tag["name"]] = tag["value"]
+
+    return tags
 
 
-def _remove_metadata(featurestore_id, featuregroup_id, name):
+def _remove_tag(featurestore_id, id, tag, resource):
     """
-    Makes a REST call to Hopsworks to delete extended metadata attached to a featuregroup
+    Makes a REST call to Hopsworks to delete tags attached to a featuregroup or training dataset
 
     Args:
         :featurestore_id: the id of the featurestore
-        :featuregroup_id: the id of the featuregroup
-        :name: the name of the extended attribute
+        :id: the id of the featuregroup or training dataset
+        :tag: name of the tag
+        :resource: featuregroup or training dataset resource
 
     Returns:
         None
 
     """
     method = constants.HTTP_CONFIG.HTTP_DELETE
-    headers = {constants.HTTP_CONFIG.HTTP_CONTENT_TYPE: constants.HTTP_CONFIG.HTTP_APPLICATION_JSON}
     resource_url = (constants.DELIMITERS.SLASH_DELIMITER +
                     constants.REST_CONFIG.HOPSWORKS_REST_RESOURCE + constants.DELIMITERS.SLASH_DELIMITER +
                     constants.REST_CONFIG.HOPSWORKS_PROJECT_RESOURCE + constants.DELIMITERS.SLASH_DELIMITER +
                     hdfs.project_id() + constants.DELIMITERS.SLASH_DELIMITER +
                     constants.REST_CONFIG.HOPSWORKS_FEATURESTORES_RESOURCE + constants.DELIMITERS.SLASH_DELIMITER +
                     str(featurestore_id) + constants.DELIMITERS.SLASH_DELIMITER +
-                    constants.REST_CONFIG.HOPSWORKS_FEATUREGROUPS_RESOURCE +
-                    constants.DELIMITERS.SLASH_DELIMITER + str(featuregroup_id) + constants.DELIMITERS.SLASH_DELIMITER +
-                    constants.REST_CONFIG.HOPSWORKS_FEATUREGROUPS_XATTRS_RESOURCE + constants.DELIMITERS.SLASH_DELIMITER +
-                    name)
-    response = util.send_request(method, resource_url, headers=headers)
-    if response.status_code == 404:
-        raise RestAPIError("Could not remove extened attributes from a featuregroup (url: {}), server response: \n " \
-                           "HTTP code: {}, HTTP reason: {}".format(resource_url, response.status_code, response.reason))
+                    resource +
+                    constants.DELIMITERS.SLASH_DELIMITER + str(id) + constants.DELIMITERS.SLASH_DELIMITER +
+                    constants.REST_CONFIG.HOPSWORKS_FEATURESTORE_TAGS_RESOURCE + constants.DELIMITERS.SLASH_DELIMITER +
+                    tag)
+    response = util.send_request(method, resource_url)
     if response.status_code >= 400:
         response_object = response.json()
         error_code, error_msg, user_msg = util._parse_rest_error(response_object)
-        raise RestAPIError("Could not remove extened attributes from a featuregroup (url: {}), server response: \n " \
+        raise RestAPIError("Could not remove tags (url: {}), server response: \n " \
                            "HTTP code: {}, HTTP reason: {}, error code: {}, error msg: {}, user msg: {}".format(
             resource_url, response.status_code, response.reason, error_code, error_msg, user_msg))
+
+
+def _get_fs_tags():
+    """
+    Makes a REST call to Hopsworks to get tags that can be attached to featuregroups or training datasets
+
+    Returns:
+        List of tags
+
+    """
+    method = constants.HTTP_CONFIG.HTTP_GET
+    headers = {constants.HTTP_CONFIG.HTTP_CONTENT_TYPE: constants.HTTP_CONFIG.HTTP_APPLICATION_JSON}
+    resource_url = (constants.DELIMITERS.SLASH_DELIMITER +
+                    constants.REST_CONFIG.HOPSWORKS_REST_RESOURCE +
+                    constants.DELIMITERS.SLASH_DELIMITER +
+                    constants.REST_CONFIG.HOPSWORKS_FEATURESTORE_TAGS_RESOURCE)
+    response = util.send_request(method, resource_url, headers=headers)
+    response_object = response.json()
+    if response.status_code >= 400:
+        error_code, error_msg, user_msg = util._parse_rest_error(response_object)
+        raise RestAPIError("Could not get tags (url: {}), server response: \n " \
+                           "HTTP code: {}, HTTP reason: {}, error code: {}, error msg: {}, user msg: {}".format(
+            resource_url, response.status_code, response.reason, error_code, error_msg, user_msg))
+
+    results = []
+    if 'items' in response_object:
+        for tag in response_object['items']:
+            results.append(tag['name'])
+    return results
+
+def _get_hostname():
+    """
+    Makes a REST call to Hopsworks to get hostname of the Hopsworks machine
+
+    Returns:
+        The hostname
+    """
+
+    resource_url= (constants.DELIMITERS.SLASH_DELIMITER +
+                 constants.REST_CONFIG.HOPSWORKS_REST_RESOURCE + constants.DELIMITERS.SLASH_DELIMITER +
+                 constants.REST_CONFIG.HOPSWORKS_VARIABLES_RESOURCE + constants.DELIMITERS.SLASH_DELIMITER +
+                 constants.REST_CONFIG.HOPSWORKS_ENDPOINT)
+    response = util.send_request(constants.HTTP_CONFIG.HTTP_GET, resource_url)
+    if (response.status_code // 100) != 2:
+        error_code, error_msg, user_msg = "", "", ""
+
+        raise RestAPIError("Could not execute HTTP request (url: {}), server response: \n "
+                           "HTTP code: {}, HTTP reason: {}, error code: {}, error msg: {}, user msg: {}".format(
+            resource_url, response.status_code, response.reason, error_code, error_msg, user_msg))
+
+    return response.json()['successMessage'].split(':')[0]
\ No newline at end of file
