@@ -7,17 +7,13 @@ import json
 import numpy as np
 from hops.featurestore_impl.exceptions.exceptions import InferTFRecordSchemaError, \
     InvalidPrimaryKey, SparkToHiveSchemaConversionError, CouldNotConvertDataframe, \
-    SparkToMySQLSchemaConversionError, FeatureNotFound
+    SparkToMySQLSchemaConversionError, FeatureNotFound, FeaturegroupNotFound, \
+    TrainingDatasetNotFound
 import pandas as pd
 import math
 import re
 import os
 
-try:
-    import pydoop.hdfs as pydoop
-except:
-    pass
-
 # for backwards compatibility
 try:
     import h5py
@@ -60,7 +56,8 @@ def _do_get_latest_training_dataset_version(training_dataset_name, featurestore_
     if (len(versions) > 0):
         return max(versions)
     else:
-        return 0;
+        raise TrainingDatasetNotFound("There was no version for training dataset {} "
+                               "found".format(training_dataset_name))
 
 
 def _get_spark_array_size(spark_df, array_col_name):
@@ -101,14 +98,14 @@ def _get_dataframe_tf_record_schema_json(spark_df, fixed=True):
     example_json = {}
     for col in spark_df.dtypes:
         if col[1] in constants.FEATURE_STORE.TF_RECORD_INT_SPARK_TYPES:
-            example[str(col[0])] = tf.FixedLenFeature([], tf.int64)
+            example[str(col[0])] = tf.io.FixedLenFeature([], tf.int64)
             example_json[str(col[0])] = {
                 constants.FEATURE_STORE.TF_RECORD_SCHEMA_FEATURE:
                     constants.FEATURE_STORE.TF_RECORD_SCHEMA_FEATURE_FIXED,
                 constants.FEATURE_STORE.TF_RECORD_SCHEMA_TYPE: constants.FEATURE_STORE.TF_RECORD_INT_TYPE}
-            tf.FixedLenFeature([], tf.int64)
+            tf.io.FixedLenFeature([], tf.int64)
         if col[1] in constants.FEATURE_STORE.TF_RECORD_FLOAT_SPARK_TYPES:
-            example[str(col[0])] = tf.FixedLenFeature([], tf.float32)
+            example[str(col[0])] = tf.io.FixedLenFeature([], tf.float32)
             example_json[str(col[0])] = {
                 constants.FEATURE_STORE.TF_RECORD_SCHEMA_FEATURE:
                     constants.FEATURE_STORE.TF_RECORD_SCHEMA_FEATURE_FIXED,
@@ -116,7 +113,7 @@ def _get_dataframe_tf_record_schema_json(spark_df, fixed=True):
         if col[1] in constants.FEATURE_STORE.TF_RECORD_INT_ARRAY_SPARK_TYPES:
             if fixed:
                 array_len = _get_spark_array_size(spark_df, str(col[0]))
-                example[str(col[0])] = tf.FixedLenFeature(shape=[array_len], dtype=tf.int64)
+                example[str(col[0])] = tf.io.FixedLenFeature(shape=[array_len], dtype=tf.int64)
                 example_json[str(col[0])] = {
                     constants.FEATURE_STORE.TF_RECORD_SCHEMA_FEATURE:
                         constants.FEATURE_STORE.TF_RECORD_SCHEMA_FEATURE_FIXED,
@@ -124,7 +121,7 @@ def _get_dataframe_tf_record_schema_json(spark_df, fixed=True):
                     constants.FEATURE_STORE.TF_RECORD_SCHEMA_SHAPE: [array_len]
                 }
             else:
-                example[str(col[0])] = tf.VarLenFeature(tf.int64)
+                example[str(col[0])] = tf.io.VarLenFeature(tf.int64)
                 example_json[str(col[0])] = {
                     constants.FEATURE_STORE.TF_RECORD_SCHEMA_FEATURE:
                         constants.FEATURE_STORE.TF_RECORD_SCHEMA_FEATURE_VAR,
@@ -132,7 +129,7 @@ def _get_dataframe_tf_record_schema_json(spark_df, fixed=True):
         if col[1] in constants.FEATURE_STORE.TF_RECORD_FLOAT_ARRAY_SPARK_TYPES:
             if fixed:
                 array_len = _get_spark_array_size(spark_df, str(col[0]))
-                example[str(col[0])] = tf.FixedLenFeature(shape=[array_len], dtype=tf.float32)
+                example[str(col[0])] = tf.io.FixedLenFeature(shape=[array_len], dtype=tf.float32)
                 example_json[str(col[0])] = {
                     constants.FEATURE_STORE.TF_RECORD_SCHEMA_FEATURE:
                         constants.FEATURE_STORE.TF_RECORD_SCHEMA_FEATURE_FIXED,
@@ -140,14 +137,14 @@ def _get_dataframe_tf_record_schema_json(spark_df, fixed=True):
                     constants.FEATURE_STORE.TF_RECORD_SCHEMA_SHAPE: [array_len]
                 }
             else:
-                example[str(col[0])] = tf.VarLenFeature(tf.float32)
+                example[str(col[0])] = tf.io.VarLenFeature(tf.float32)
                 example_json[str(col[0])] = {
                     constants.FEATURE_STORE.TF_RECORD_SCHEMA_FEATURE:
                         constants.FEATURE_STORE.TF_RECORD_SCHEMA_FEATURE_VAR,
                     constants.FEATURE_STORE.TF_RECORD_SCHEMA_TYPE: constants.FEATURE_STORE.TF_RECORD_FLOAT_TYPE}
         if col[1] in constants.FEATURE_STORE.TF_RECORD_STRING_ARRAY_SPARK_TYPES or col[
             1] in constants.FEATURE_STORE.TF_RECORD_STRING_SPARK_TYPES:
-            example[str(col[0])] = tf.VarLenFeature(tf.string)
+            example[str(col[0])] = tf.io.VarLenFeature(tf.string)
             example_json[str(col[0])] = {
                 constants.FEATURE_STORE.TF_RECORD_SCHEMA_FEATURE: constants.FEATURE_STORE.TF_RECORD_SCHEMA_FEATURE_VAR,
                 constants.FEATURE_STORE.TF_RECORD_SCHEMA_TYPE: constants.FEATURE_STORE.TF_RECORD_STRING_TYPE}
@@ -176,41 +173,41 @@ def _convert_tf_record_schema_json_to_dict(tf_record_json_schema):
                 constants.FEATURE_STORE.TF_RECORD_SCHEMA_FEATURE_FIXED \
                 and value[constants.FEATURE_STORE.TF_RECORD_SCHEMA_TYPE] == constants.FEATURE_STORE.TF_RECORD_INT_TYPE:
             if constants.FEATURE_STORE.TF_RECORD_SCHEMA_SHAPE in value:
-                example[str(key)] = tf.FixedLenFeature(shape=value[constants.FEATURE_STORE.TF_RECORD_SCHEMA_SHAPE],
+                example[str(key)] = tf.io.FixedLenFeature(shape=value[constants.FEATURE_STORE.TF_RECORD_SCHEMA_SHAPE],
                                                        dtype=tf.int64)
             else:
-                example[str(key)] = tf.FixedLenFeature([], tf.int64)
+                example[str(key)] = tf.io.FixedLenFeature([], tf.int64)
         if value[
             constants.FEATURE_STORE.TF_RECORD_SCHEMA_FEATURE] == \
                 constants.FEATURE_STORE.TF_RECORD_SCHEMA_FEATURE_FIXED and \
                         value[constants.FEATURE_STORE.TF_RECORD_SCHEMA_TYPE] == \
                         constants.FEATURE_STORE.TF_RECORD_FLOAT_TYPE:
             if constants.FEATURE_STORE.TF_RECORD_SCHEMA_SHAPE in value:
-                example[str(key)] = tf.FixedLenFeature(shape=value[constants.FEATURE_STORE.TF_RECORD_SCHEMA_SHAPE],
+                example[str(key)] = tf.io.FixedLenFeature(shape=value[constants.FEATURE_STORE.TF_RECORD_SCHEMA_SHAPE],
                                                        dtype=tf.float32)
             else:
-                example[str(key)] = tf.FixedLenFeature([], tf.float32)
+                example[str(key)] = tf.io.FixedLenFeature([], tf.float32)
         if value[
             constants.FEATURE_STORE.TF_RECORD_SCHEMA_FEATURE] == \
                 constants.FEATURE_STORE.TF_RECORD_SCHEMA_FEATURE_VAR and \
                         value[
                             constants.FEATURE_STORE.TF_RECORD_SCHEMA_TYPE] == \
                         constants.FEATURE_STORE.TF_RECORD_INT_TYPE:
-            example[str(key)] = tf.VarLenFeature(tf.int64)
+            example[str(key)] = tf.io.VarLenFeature(tf.int64)
         if value[
             constants.FEATURE_STORE.TF_RECORD_SCHEMA_FEATURE] == \
                 constants.FEATURE_STORE.TF_RECORD_SCHEMA_FEATURE_VAR and \
                         value[
                             constants.FEATURE_STORE.TF_RECORD_SCHEMA_TYPE] == \
                         constants.FEATURE_STORE.TF_RECORD_FLOAT_TYPE:
-            example[str(key)] = tf.VarLenFeature(tf.float32)
+            example[str(key)] = tf.io.VarLenFeature(tf.float32)
         if value[
             constants.FEATURE_STORE.TF_RECORD_SCHEMA_FEATURE] == \
                 constants.FEATURE_STORE.TF_RECORD_SCHEMA_FEATURE_VAR and \
                         value[
                             constants.FEATURE_STORE.TF_RECORD_SCHEMA_TYPE] == \
                         constants.FEATURE_STORE.TF_RECORD_STRING_TYPE:
-            example[str(key)] = tf.VarLenFeature(tf.string)
+            example[str(key)] = tf.io.VarLenFeature(tf.string)
     return example
 
 
@@ -262,7 +259,8 @@ def _do_get_latest_featuregroup_version(featuregroup_name, featurestore_metadata
     if (len(versions) > 0):
         return max(versions)
     else:
-        return 0
+        raise FeaturegroupNotFound("There was no version for featuregroup {} "
+                               "found".format(featuregroup_name))
 
 
 def _get_default_primary_key(featuregroup_df):
@@ -1061,106 +1059,36 @@ def _get_spark_sql_catalog_impl(spark):
     return dict(spark.sparkContext._conf.getAll())[constants.SPARK_CONFIG.SPARK_SQL_CATALOG_IMPLEMENTATION]
 
 
-def _get_on_demand_featuregroup_type_info(featurestore_metadata):
-    """
-    Gets the type information for an on-demand feature group that the backend expects
-
-    Args:
-         :featurestore_metadata: metadata of the featurestore
-
-    Returns:
-        the type information of the feature group, tuple of (type, dtotype)
-    """
-    featuregroup_type = featurestore_metadata.settings.on_demand_featuregroup_type
-    featuregroup_type_dto = featurestore_metadata.settings.on_demand_featuregroup_dto_type
-    return featuregroup_type, featuregroup_type_dto
-
-
-def _get_cached_featuregroup_type_info(featurestore_metadata):
-    """
-    Gets the type information for a cached feature group that the backend expects
-
-    Args:
-         :featurestore_metadata: metadata of the featurestore
-
-    Returns:
-        the type information of the feature group, tuple of (type, dtotype)
-    """
-    featuregroup_type = featurestore_metadata.settings.cached_featuregroup_type
-    featuregroup_type_dto = featurestore_metadata.settings.cached_featuregroup_dto_type
-    return featuregroup_type, featuregroup_type_dto
-
-
 def _get_training_dataset_type_info(featurestore_metadata, external=False):
     """
-    Gets the type information of a training datasetthat the backend expects
+    Gets the type information of a training dataset that the backend expects
 
     Args:
          :featurestore_metadata: metadata of the featurestore
          :external: whether it is an external featuregroup or not
 
     Returns:
-        the type information of the training dataset, tuple of (type, dtotype)
+        the type information of the training dataset
     """
     if external:
         training_dataset_type = featurestore_metadata.settings.external_training_dataset_type
-        training_dataset_type_dto = featurestore_metadata.settings.external_training_dataset_dto_type
     else:
         training_dataset_type = featurestore_metadata.settings.hopsfs_training_dataset_type
-        training_dataset_type_dto = featurestore_metadata.settings.hopsfs_training_dataset_dto_type
-    return training_dataset_type, training_dataset_type_dto
+    return training_dataset_type
 
 
-def _get_hopsfs_training_dataset_path(training_dataset_name, hdfs_store_path, data_format):
+def _get_external_training_dataset_path(path):
     """
-    Utility function for getting the hopsfs path of a training dataset in the feature store
+    The location Hopsworks returns cointain the filesystem which is set to s3://. Spark expects the filesystem to be s3a:// 
 
     Args:
-        :training_dataset_name: name of the training dataset
-        :hdfs_store_path: the hdfs path to the dataset where all the training datasets are stored
-        :data_format: data format of the training datataset
-
-    Return:
-        the hdfs path to the training dataset
-    """
-    hdfs_path = hdfs_store_path + \
-                constants.DELIMITERS.SLASH_DELIMITER + training_dataset_name
-    if data_format == constants.FEATURE_STORE.TRAINING_DATASET_IMAGE_FORMAT:
-        hdfs_path = hdfs_store_path
-    # abspath means "hdfs://namenode:port/ is preprended
-    path = util.abspath(hdfs_path)
-    return path
-
-
-def _get_external_training_dataset_path(training_dataset_name, training_dataset_version, bucket, path):
-    """
-    Utility function for getting the S3 path of a training dataset in the feature store
-
-    Args:
-        :training_dataset_name: name of the training dataset
-        :training_dataset_version: version of the training dataset
-        :bucket: the s3 bucket
-        :path: user-supplied path
+        path: the path returned by Hopsworks
 
     Returns:
-        the s3 path to the training dataset
+        the s3 path to read the training dataset from Spark 
     """
-    if path is None or path is "":
-        path = ""
-        if constants.S3_CONFIG.S3_FILE_PREFIX not in bucket:
-            path = constants.S3_CONFIG.S3_FILE_PREFIX
-        path = path + bucket + constants.DELIMITERS.SLASH_DELIMITER + constants.S3_CONFIG.S3_TRAINING_DATASETS_FOLDER \
-               + constants.DELIMITERS.SLASH_DELIMITER + _get_table_name(training_dataset_name, training_dataset_version)
-    else:
-        if constants.S3_CONFIG.S3_FILE_PREFIX not in bucket:
-            path = constants.S3_CONFIG.S3_FILE_PREFIX + bucket + constants.DELIMITERS.SLASH_DELIMITER + path
-        else:
-            path = bucket + constants.DELIMITERS.SLASH_DELIMITER + path
-        path = path + constants.DELIMITERS.SLASH_DELIMITER + _get_table_name(training_dataset_name,
-                                                                             training_dataset_version)
-    return path
-
 
+    return path.replace("s3", "s3a", 1)
 
 
 def _setup_s3_credentials_for_spark(access_key, secret_key, spark):
