@@ -11,13 +11,18 @@ import os
 import errno
 
 from hops import constants
+from hops.service_discovery import ServiceDiscovery
 import sys
 import subprocess
 
+import ntpath
+import importlib
+
 # Compatibility with SageMaker
 try:
     import pydoop.hdfs as hdfs
     import pydoop.hdfs.path as path
+    import pydoop.hdfs.fs as hdfs_fs
 except:
     pass
 
@@ -27,6 +32,34 @@ from xml.dom import minidom
 tls_enabled = None
 webhdfs_address = None
 
+
+# Replace Pydoop split method to be able to support hopsfs:// schemes
+class _HopsFSPathSplitter(hdfs.path._HdfsPathSplitter):
+    @classmethod
+    def split(cls, hdfs_path, user):
+        if not hdfs_path:
+            cls.raise_bad_path(hdfs_path, "empty")
+        scheme, netloc, path = cls.parse(hdfs_path)
+        if not scheme:
+            scheme = "file" if hdfs_fs.default_is_local() else "hdfs"
+        if scheme == "hdfs" or scheme == "hopsfs":
+            if not path:
+                cls.raise_bad_path(hdfs_path, "path part is empty")
+            if ":" in path:
+                cls.raise_bad_path(
+                    hdfs_path, "':' not allowed outside netloc part"
+                )
+            hostname, port = cls.split_netloc(netloc)
+            if not path.startswith("/"):
+                path = "/user/%s/%s" % (user, path)
+        elif scheme == "file":
+            hostname, port, path = "", 0, netloc + path
+        else:
+            cls.raise_bad_path(hdfs_path, "unsupported scheme %r" % scheme)
+        return hostname, port, path
+
+hdfs.path._HdfsPathSplitter = _HopsFSPathSplitter
+
 def get_plain_path(abs_path):
     """
     Convert absolute HDFS/HOPSFS path to plain path (dropping prefix and ip)
@@ -795,10 +828,11 @@ def add_module(hdfs_path, project=None):
         py_path = copy_to_local(hdfs_path, localized_deps, overwrite=True)
         if py_path not in sys.path:
             sys.path.append(py_path)
+        _reload(py_path)
         return py_path
     elif path.isfile(hdfs_path) and hdfs_path.endswith('.ipynb'):
         ipynb_path = copy_to_local(hdfs_path, localized_deps, overwrite=True)
-        python_path = os.environ['PYSPARK_PYTHON']
+        python_path = sys.executable
         jupyter_binary = os.path.dirname(python_path) + '/jupyter'
         if not os.path.exists(jupyter_binary):
             raise Exception('Could not find jupyter binary on path {}'.format(jupyter_binary))
@@ -816,10 +850,18 @@ def add_module(hdfs_path, project=None):
             raise Exception('Could not find converted .py file on path {}'.format(converted_py_path))
         if converted_py_path not in sys.path:
             sys.path.append(converted_py_path)
+        _reload(converted_py_path)
         return converted_py_path
     else:
         raise Exception("Given path " + hdfs_path + " does not point to a .py or .ipynb file")
 
+def _reload(path):
+    try:
+        module_name = ntpath.basename(path).split(".")[0]
+        imported_module = importlib.import_module(module_name)
+        importlib.reload(imported_module)
+    except Exception as err:
+        print('Failed to automatically reload module on path {} with exception: {}'.format(path, err))
 
 def is_tls_enabled():
     """
@@ -839,42 +881,33 @@ def is_tls_enabled():
                 tls_enabled = item.getElementsByTagName("value")[0].firstChild.data == 'true'
     return tls_enabled
 
-
 def _get_webhdfs_address():
     """
-    Reads the ipc.server.ssl.enabled property from core-site.xml.
+    Makes an SRV DNS query to get the target and port of NameNode's web interface
 
     Returns:
-        returns True if ipc.server.ssl.enabled is true. False otherwise.
+        returns webhdfs endpoint
     """
     global webhdfs_address
     if webhdfs_address is None:
-        hadoop_conf_path = os.environ['HADOOP_CONF_DIR']
-        webhdfs_config = "dfs.namenode.http-address"
-        if is_tls_enabled():
-            webhdfs_config = "dfs.namenode.https-address"
-        for item in minidom.parse(os.path.join(hadoop_conf_path,'hdfs-site.xml')).getElementsByTagName('property'):
-            name = item.getElementsByTagName("name")[0]
-            if name.firstChild.data == webhdfs_config:
-                webhdfs_address = item.getElementsByTagName("value")[0].firstChild.data
+        _, port = ServiceDiscovery.get_any_service('http.namenode')
+        webhdfs_address = ServiceDiscovery.construct_service_fqdn('http.namenode') + ":" + str(port)
     return webhdfs_address
-
-
+        
 def get_webhdfs_host():
     """
-    Reads the ipc.server.ssl.enabled property from core-site.xml and returns webhdfs hostname.
+    Makes an SRV DNS query and gets the actual hostname of the NameNode
 
     Returns:
-        returns the webhdfs hostname.
+        returns NameNode's hostname
     """
-    return socket.gethostbyaddr(_get_webhdfs_address().split(":")[0])[0]
-
+    return ServiceDiscovery.construct_service_fqdn('http.namenode')
 
 def get_webhdfs_port():
     """
-    Reads the ipc.server.ssl.enabled property from core-site.xml and the webhdfs port.
+    Makes an SRV DNS query and gets NameNode's port for WebHDFS
 
     Returns:
-        returns the webhdfs port.
+        returns NameNode's port for WebHDFS
     """
     return _get_webhdfs_address().split(":")[1]
